@@ -1656,15 +1656,7 @@ int json_unescape(CHARSET_INFO *json_cs,
     }
     if (c_len == MY_CS_ILUNI)
     {
-      /*
-        Result charset doesn't support the json's character.
-        Let's replace it with the '?' symbol.
-      */
-      if ((c_len= my_ci_wc_mb(res_cs, '?', res, res_end)) > 0)
-      {
-        res+= c_len;
-        continue;
-      }
+      return -1;
     }
     /* Result buffer is too small. */
     return -1;
@@ -1706,7 +1698,7 @@ static enum json_esc_char_classes json_escape_chr_map[0x60] = {
 };
 
 
-static const char hexconv[16] = "0123456789ABCDEF";
+static const char hexconv[17] = "0123456789ABCDEF";
 
 
 int json_escape(CHARSET_INFO *str_cs,
@@ -2021,7 +2013,43 @@ enum json_types json_get_object_nkey(const char *js __attribute__((unused)),
                                      const char **value __attribute__((unused)),
                                      int *value_len __attribute__((unused)))
 {
-  return JSV_NOTHING;
+  json_engine_t je;
+  int keys_found= 0;
+
+  json_scan_start(&je, &my_charset_utf8mb4_bin,(const uchar *) js,
+                  (const uchar *) js_end);
+
+  if (json_read_value(&je) ||
+      je.value_type != JSON_VALUE_OBJECT)
+    goto err_return;
+
+  while (!json_scan_next(&je))
+  {
+    switch (je.state)
+    {
+    case JST_KEY:
+      if (nkey == keys_found)
+      {
+        *keyname= (char *) je.s.c_str;
+        while (json_read_keyname_chr(&je) == 0)
+          *keyname_end= (char *) je.s.c_str;
+
+        return smart_read_value(&je, value, value_len);
+      }
+
+      keys_found++;
+      if (json_skip_key(&je))
+        goto err_return;
+
+      break;
+
+    case JST_OBJ_END:
+      return JSV_NOTHING;
+    }
+  }
+
+err_return:
+  return JSV_BAD_JSON;
 }
 
 
